/*
	JAHIDElement.h
	
	Abstract class representing an IOKit HID device element. Different
	implementations are used on different version of Mac OS X.
	
	
	Copyright © 2008 Jens Ayton

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the “Software”),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:
	
	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
	THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/

#import <JAHIDDevices/JAHIDElementCollection.h>


#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5
/*	In 10.4 and earlier SDKs, IOHIDElementFlags is declared only as an enum
	tag. In 10.5 SDK, it's declared only as a typedef.
*/
typedef uint32_t IOHIDElementFlags;
#define kIOHIDElementFlagsKey                       "Flags"
#endif


enum
{
	kJAHIDFlagIsArray			= 0x0001UL,
	kJAHIDFlagIsRelative		= 0x0002UL,
	kJAHIDFlagIsWrapping		= 0x0004UL,
	kJAHIDFlagIsNonLinear		= 0x0008UL,
	kJAHIDFlagHasPreferredState	= 0x0010UL,
	kJAHIDFlagHasNullState		= 0x0020UL
};
typedef uint_fast16_t JAHIDElementFlags;


@interface JAHIDElement: JAHIDElementCollection

- (JAHIDElement *) parentElement;	// Same as parent

- (NSString *) name;

- (JAHIDElementCookie) cookie;

- (IOHIDElementType) type;
- (IOHIDElementCollectionType) collectionType;
- (JAHIDUsagePage) usagePage;
- (JAHIDUsage) usage;

- (JAInputInt) minimum;
- (JAInputInt) maximum;
- (JAInputInt) scaledMinimum;
- (JAInputInt) scaledMaximum;

- (BOOL) isArray;
- (BOOL) isRelative;
- (BOOL) isWrapping;
- (BOOL) isNonLinear;
- (BOOL) hasPreferredState;
- (BOOL) hasNullState;

// Compact equivalent to isFoo methods above
- (JAHIDElementFlags) flags;

- (JAHIDUnit) unit;
- (JAHIDUnitExponent) unitExponent;

@end


@interface JAHIDElement (SearchingAndMatching)

- (BOOL) matchesDictionary:(NSDictionary *)dict;

@end
