/*
	JAHIDElementCollection.h
	
	Abstract class representing a collection of HID elements, or more
	specfically a node in a tree of HID objects. The root of the tree is a
	JAHIDDevice. All other nodes are JAHIDElements.
	
	JAHIDElementCollections are immutable. The properties of individual items
	in the collection may change, but the elements in a collection cannot.
	
	
	Copyright © 2008 Jens Ayton

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the “Software”),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:
	
	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
	THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/

#import <IOKit/hid/IOHIDKeys.h>
#import <IOKit/hid/IOHIDUsageTables.h>
#import <JAHIDDevices/JAHIDDevicesBaseTypes.h>

@class JAHIDDevice, JAHIDElement;


@interface JAHIDElementCollection: NSObject <NSCopying>

- (JAHIDElementCollection *) parent;
- (JAHIDDevice *) device;

- (NSArray *) elements;	// Values are JAHIDElements.

/*	Takes keys from IOHIDKeys.h. Convenience accessors for most properties are
	provided by JAHIDDevice and JAHIDElement. */
- (id) valueForHIDKey:(NSString *)key;

@end


typedef BOOL (*JAHIDElementMatchingPredicate)(JAHIDElement *element, void *info);


// Convenience methods: searching for categories of elements
@interface JAHIDElementCollection (SearchingAndMatching)

- (NSEnumerator *) elementEnumerator;

- (NSArray *) elementsMatchingDictionary:(NSDictionary *)dict
							 recursively:(BOOL)recursively;
- (NSArray *) elementsMatchingPredicate:(JAHIDElementMatchingPredicate)predicate
								   info:(void *)info
							recursively:(BOOL)recursively;

- (NSArray *) collectionsRecursively:(BOOL)recursively;
- (NSArray *) applicationCollectionsRecursively:(BOOL)recursively;
- (NSArray *) physicalCollectionsRecursively:(BOOL)recursively;
- (NSArray *) logicalCollectionsRecursively:(BOOL)recursively;

- (NSArray *) elementsWithUsagePage:(JAHIDUsagePage)usagePage
						recursively:(BOOL)recursively;
- (NSArray *) elementsWithUsagePage:(JAHIDUsagePage)usagePage
							  usage:(JAHIDUsage)usage
						recursively:(BOOL)recursively;

- (JAHIDElement *) firstElementMatchingDictionary:(NSDictionary *)dict
									  recursively:(BOOL)recursively;
- (JAHIDElement *) firstElementMatchingPredicate:(JAHIDElementMatchingPredicate)predicate
											info:(void *)info
									 recursively:(BOOL)recursively;

- (JAHIDElement *) firstElementWithUsagePage:(JAHIDUsagePage)usagePage
								 recursively:(BOOL)recursively;
- (JAHIDElement *) firstElementWithUsagePage:(JAHIDUsagePage)usagePage
									   usage:(JAHIDUsage)usage
								 recursively:(BOOL)recursively;

@end
