/*
	JAInputDevicesBaseTypes.h
	
	Copyright © 2008 Jens Ayton

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the “Software”),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:
	
	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
	THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/

#ifndef INCLUDED_JAINPUTDEVICESBASETYPES_h
#define INCLUDED_JAINPUTDEVICESBASETYPES_h


#if 0	// Dump target info to build log
#if __ppc__
#warning PPC
#elif __ppc64__
#warning PPC_64
#elif __i386__
#warning i386
#elif __x86_64__
#warning x86_64
#else
#warning Unidentified target.
#endif


#if MAC_OS_X_VERSION_MAX_ALLOWED == MAC_OS_X_VERSION_10_5
#warning Deployment target 10.5
#elif MAC_OS_X_VERSION_MAX_ALLOWED == MAC_OS_X_VERSION_10_4
#warning Deployment target 10.4
#elif MAC_OS_X_VERSION_MAX_ALLOWED == MAC_OS_X_VERSION_10_3
#warning Deployment target 10.3
#else
#warning Deployment target unknown
#endif
#endif // 0


#ifndef JAINPUT_OBJC
#if __OBJC__
/*! @parseOnly */
#define JAINPUT_OBJC			1
#else
/*! @parseOnly */
#define JAINPUT_OBJC			0
#endif
#endif

#if JAINPUT_OBJ
#ifndef JAINPUT_OBJC_FRAGILE
#if __LP64__
/*! @parseOnly */
#define JAINPUT_OBJC_FRAGILE	0
#else
/*! @parseOnly */
#define JAINPUT_OBJC_FRAGILE	1
#endif
#endif

#endif	/* JAINPUT_OBJC */


// Basic C++-cleanness.
#ifdef __cplusplus
/*! @parseOnly */
#define JAINPUT_NOT_CPP  extern "C" {
/*! @parseOnly */
#define JAINPUT_END_NOT_CPP }
#else
/*! @parseOnly */
#define JAINPUT_NOT_CPP
/*! @parseOnly */
#define JAINPUT_END_NOT_CPP
#endif


/*	Declare JAInputInt and JAInputUInt.
	These are equivalent to NSInteger/NSUInteger and (for JAInputInt) CFIndex:
	32-bit on 32-bit systems, 64-bit on 64-bit systems.
*/
#if JAINPUT_OBJC
#import <Foundation/Foundation.h>
#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
/*!	@typedef	JAInputInt
	@abstract	General signed integer type, appropriately sized for
				architecture (32 or 64 bits).
	@discussion	<code>JAInputInt</code> is equivalent to <code>NSInteger</code>.
				If included in an Objective-C file and building for Mac OS X
				10.5 or later, <code>NSInteger</code> will be used. On earlier
				systems, <code>int</code> is used. In non-Objective-C code,
				<code>CFIndex</code> is used.
				
				In all cases, this is equivalent to <code>long</code>. The
				complexity is designed to match <code>NSInteger</code>’s
				behaviour with respect to Objective-C type encoding strings.
*/

typedef NSInteger JAInputInt;

/*!	@typedef	JAInputUInt
	@abstract	General unsigned integer type, appropriately sized for
				architecture (32 or 64 bits).
	@discussion	<code>JAInputUInt</code> is equivalent to
				<code>NSYInteger</code>. If included in an Objective-C file
				and building for Mac OS X 10.5 or later, <code>NSUInteger</code>
				will be used. On earlier systems and in 32-bit non-Objective-C
				code, <code>unsigned int</code> is used. In 64-bit
				non-Objective-C code, <code>unsigned long</code> is used.
				
				In all cases, this is equivalent to <code>unsigned long</code>.
				The complexity is designed to match <code>NSUInteger</code>’s
				behaviour with respect to Objective-C type encoding strings.
*/
typedef NSUInteger JAInputUInt;

/*!	@const		JAInputIntMax
	@abstract	Maximum value for JAInputInt.
*/
#define JAInputIntMax NSIntegerMax

/*!	@const		NSIntegerMin
	@abstract	Minimum value for JAInputInt.
*/
#define JAInputIntMin NSIntegerMin

/*!	@const		NSUIntegerMax
	@abstract	Maximum value for JAInputUInt.
*/
#define JAInputUIntMax NSUIntegerMax
#else
typedef int JAInputInt;
typedef unsigned int JAInputUInt;
#define JAInputIntMax INT_MAX
#define JAInputIntMin INT_MIN
#define JAInputUIntMax UINT_MAX
#endif
#else	// Not ObjC
#include <CoreFoundation/CoreFoundation.h>
typedef CFIndex JAInputInt;
typedef unsigned long JAInputUInt;
#define JAInputIntMax LONG_MAX
#define JAInputIntMin LONG_MIN
#define JAInputUIntMax ULONG_MAX
#endif


typedef JAInputUInt JAInputTickCount;

typedef uint_fast64_t	JAInputBitField;


// Foundation/CoreFoundation types
#if JAINPUT_OBJC

typedef NSString		*JAInputStringRef;
typedef NSArray			*JAInputArrayRef;
typedef NSDictionary	*JAInputDictionaryRef;
typedef NSSet			*JAInputSetRef;
typedef BOOL			JAInputBoolean;

#else

typedef CFStringRef		JAInputStringRef;
typedef CFArrayRef		JAInputArrayRef;
typedef CFDictionaryRef	JAInputDictionaryRef;
typedef CFSetRef		JAInputSetRef;
typedef signed char		JAInputBoolean;

#endif


#endif	/* INCLUDED_JAINPUTDEVICESBASETYPES_h */
